package web.multitask.trismegistoservices.services;

import java.io.ByteArrayOutputStream;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xssf.usermodel.extensions.XSSFCellBorder.BorderSide;
import org.json.JSONArray;
import org.json.JSONObject;
import java.awt.Color;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;

@Service
public class ExcelService {

    public byte[] generateExcel(JSONObject json) {
        ByteArrayOutputStream outByteStream = new ByteArrayOutputStream();
        XSSFWorkbook workbook = new XSSFWorkbook();
        createSheet(workbook, json);
        try {
            workbook.write(outByteStream);
            workbook.close();
            return outByteStream.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private void createSheet(XSSFWorkbook workbook, JSONObject json) {
        if (json.optJSONArray("sheetArray", new JSONArray()).isEmpty()) {
            XSSFSheet sheet = workbook.createSheet(json.optString("sheet_name", "no_name"));
            buildSheet(workbook, sheet, json);
        }else{
            JSONArray sheetArray = json.optJSONArray("sheetArray", new JSONArray());
            for(int i = 0; i < sheetArray.length(); i++){
                JSONObject sheetObject = sheetArray.optJSONObject(i);
                XSSFSheet sheet = workbook.createSheet(sheetObject.optString("sheet_name", "no_name"));
                buildSheet(workbook, sheet, sheetObject);
            }
        }
    }

    private void buildSheet(XSSFWorkbook workbook, XSSFSheet sheet, JSONObject json) {

        JSONArray identifiers = json.optJSONArray("identifiers", new JSONArray());
        JSONArray data = json.optJSONArray("data", new JSONArray());
        JSONArray styles = json.optJSONArray("styles", new JSONArray());
        for (int i = 0; i < data.length(); i++) {
            XSSFRow row = sheet.createRow(i);
            JSONObject row_data = data.optJSONObject(i);
            for (int j = 0; j < identifiers.length(); j++) {
                sheet.autoSizeColumn(j);
                String value = row_data.optString(identifiers.getString(j), "");
                XSSFCell cell = row.createCell(j);
                JSONArray styleArray = styles.optJSONArray(i, new JSONArray());
                JSONObject style = styleArray.optJSONObject(j, new JSONObject());
                XSSFCellStyle cellStyle = createCellStyle(workbook,
                            getXSSFColor(style.optString("background", "#ffffff")),
                            getXSSFColor(style.optString("foreground", "#333333")),
                            style.optBoolean("bold", false),
                            style.optBoolean("border", true));
                cell.setCellStyle(cellStyle);
                cell.setCellValue(value);
            }
        }

    }

    private XSSFCellStyle createCellStyle(XSSFWorkbook workbook, XSSFColor background, XSSFColor foreground,
            boolean bold, boolean border) {
        XSSFCellStyle style = workbook.createCellStyle();
        Font font = workbook.createFont();
        style.setAlignment(HorizontalAlignment.CENTER);
        style.setWrapText(true);
        style.setVerticalAlignment(VerticalAlignment.CENTER);
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);

        if (border) {
            style.setBorderBottom(BorderStyle.THIN);
            style.setBorderTop(BorderStyle.THIN);
            style.setBorderLeft(BorderStyle.THIN);
            style.setBorderRight(BorderStyle.THIN);
            style.setBorderColor(BorderSide.BOTTOM, getXSSFColor("#333333"));
            style.setBorderColor(BorderSide.TOP, getXSSFColor("#333333"));
            style.setBorderColor(BorderSide.LEFT, getXSSFColor("#333333"));
            style.setBorderColor(BorderSide.RIGHT, getXSSFColor("#333333"));
        }

        // font.setFontHeightInPoints((short) 15);
        font.setBold(bold);
        style.setFont(font);

        if (background != null) {
            style.setFillForegroundColor(background);
        }

        if (foreground != null) {
            style.getFont().setColor(foreground);
        }

        return style;
    }

    public XSSFColor getXSSFColor(String rgb) {
        int red = Integer.parseInt(rgb.substring(1, 3), 16);
        int green = Integer.parseInt(rgb.substring(3, 5), 16);
        int blue = Integer.parseInt(rgb.substring(5, 7), 16);
        Color awtColor = new Color(red, green, blue);
        XSSFColor xssfColor = new XSSFColor();
        xssfColor
                .setRGB(new byte[] { (byte) awtColor.getRed(), (byte) awtColor.getGreen(), (byte) awtColor.getBlue() });
        return xssfColor;
    }

    public Resource convertByteToResource(byte[] data, String filename) {
        return new ByteArrayResource(data) {
            @Override
            public String getFilename() {
                return filename;
            }
        };
    }
}